/*
 * MX 68HC11 Multitasking Executive Example Program
 */
#include \mc\stdio.h
#include \mc\lib11\mx.h
#include ex.h

int error;
char *task1h, *task2h, *task3h, *mainh, *sema, *sema1, *eflag, c, i;

task1(message)
int message;
{
switch(message){
   case 1: /* Wait on sema */
            putstr("Task 1 Waiting on semaphore\n");
            swait(sema,0);
            putstr("Task 1 signalled\n");
            break;
   case 2:  putstr("Task 1 sending message 1 to main()\n");
            sendmsg(mainh,1);
            break;
   case 3:  putstr("Task 1 sending message 1 to main() after delay\n");
            sleep(20);
            sendmsg(mainh,1);
            break;
   case 4:  putstr("Task 1 signalling semaphore after 1 second delay\n");
            sleep(20);
            ssignal(sema);
            break;
   case 5:  putstr("Task 1 signalling eflag after 1 second delay\n");
            sleep(20);
            efset(eflag, 3);
            break;
   case 6: /* Wait on event flag */
            putstr("Task 1 Waiting on event flag with mask 1\n");
            printf("Task 1 signalled returned %d\n",
                    efwait(eflag,1, NO_TIMEOUT));
            break;

   default: printf("Message received value %d\n", message); 
   }
}

task2(message)
int message;
{
switch(message){
   case 0:  /* Signal sema after 3 seconds */
            sleep(60);
            putstr("Task 2: Signalling sema\n");
            ssignal(sema); break;
   case 1: /* Wait on sema */
            putstr("Task 2 Waiting on semaphore\n");
            swait(sema,0);
            putstr("Task 2 signalled\n");
            break;
   case 2: /* Send message to main() */
            putstr("Task 2 sending message 2 to main()\n");
            sendmsg(mainh,2);
            break;
   case 3: /* Wait on sema after 1 second */
            sleep(20);
            putstr("Task 2 waiting on semaphore\n"); swait(sema,0);
            putstr("Task 2 signalled\n");
            break;
   case 6: /* Wait on event flag after 1 second */
            sleep(20);
            putstr("Task 2 waiting on event flag with mask 2\n");
            printf("Task 2 signalled  value returned %d\n",
                     efwait(eflag,2,NO_TIMEOUT));
            break;
   case 7: /* Wait on event flag */
            sleep(20);
            putstr("Task 2 waiting on event flag with mask 2\n");
            printf("Task 2 signalled  value returned %d\n",
                     efwait(eflag,2,NO_TIMEOUT));
            break;
   }
}

task3(message)
int message;
{
switch(message){
   case 1: /* Wait on sema */
            putstr("Task 3 Waiting on semaphore\n");
            swait(sema,0);
            putstr("Task 3 signalled\n");
            break;
   case 2: /* Wait on sema */
            putstr("Task 3 sending message 3 to main()\n");
            sendmsg(mainh,3);
            break;
   }
}

pause()
{
printf("\nPress any key to continue...");
getch(); putch('\n');
}

demosleep()
{
char time[4];
putstr("\nTask going to sleep for 5 seconds\n");
getTOD(time);printf("\nThe time is %2d:%02d:%02d\n",time[3], time[2], time[1]);
sleep(100);  putstr("Task woken up\n");
getTOD(time);printf("\nThe time is %2d:%02d:%02d\n",time[3], time[2], time[1]);
}

demogettime()
{
char time[4];
getTOD(time);
printf("\nThe time is %2d:%02d:%02d\n",time[3], time[2], time[1]);
}

demosendmsg() 
{
putstr("\nSending message (255) to Task 1\n");
sendmsg(task1h,255);
putstr("\nMultiple messages to same task:\n");
putstr("Request Task 2 send main() a message\n"); sendmsg(task2h,2);
putstr("Request Task 3 send main() a message\n"); sendmsg(task3h,2);
putstr("Request Task 1 send main() a message\n"); sendmsg(task1h,2);
printf("\nMessages received by main() %d %d %d\n",getmsg(0),getmsg(0),getmsg(0));
}

demogetmsgt()
{
putstr("\nPerforming getmsg() with no message and 3 second timeout\n");
printf("Message received value %d\n",getmsg(60));
putstr("\nPerforming getmsg() with message available and 3 second timeout\n");
sendmsg(mainh,1);
printf("Message received value %d\n",getmsg(60));

putstr("\nPerforming getmsg() with message available after 1 second\n");
sendmsg(task1h,3);
printf("Message received value %d\n",getmsg(60));

}

demochkmsg()
{
printf("\nchkmsg() with no message waiting... returns %04x\n",chkmsg());
sendmsg(mainh,0);
printf("chkmsg() with message waiting   ... returns %04x\n",chkmsg());
flushmsg();
}

demoopensem() 
{
sema1=sopen();
printf("\nOpening semaphore, handle returned is %04x\n",sema1);
sclose(sema1);       /* Close semaphore */
}

demoswait()
{
/* open locked semaphore.  Send message to task 2 to send a signal
 * after 2 seconds.  Wait on semaphore */
putstr("\nWaiting on semaphore that will be signalled in 3 seconds\n");
sema=sopen(0);
sendmsg(task2h,0); /* Ask Task2 to signal sem */
swait(sema,0);       /* Wait on semaphore */
putstr("Semaphore received\n");

printf("\nMultiple Tasks waiting on Semaphore:\n");
printf("Request Task 2 to wait on semaphore\n"); sendmsg(task2h,1);
printf("Request Task 3 to wait on semaphore\n"); sendmsg(task3h,1);
printf("Request Task 1 to wait on semaphore\n"); sendmsg(task1h,1);
printf("\nSignalling Semaphore\n");              ssignal(sema);
printf("Signalling Semaphore again\n");          ssignal(sema);
printf("Signalling Semaphore yet again\n");      ssignal(sema);
sclose(sema);
}

demoswaitt()
{
/* Open locked semapore */
if ((sema=sopen(0))==0) printf("No semaphore available\n");
printf("\nDoing swait() with timeout occurring\n"); 
printf("\nReturn value was %d\n",swait(sema,40));

putstr("\nDoing swait() without timeout occurring\n"); ssignal(sema);
printf("\nReturn value was %d\n",swait(sema,40));

putstr("\nDoing swait() with signal after one second\n");
sendmsg(task1h,4);/* Request ssignal after one second */
printf("\nReturn value was %d\n",swait(sema,40));

pause();

/* Have multiple tasks wait on semaphore with middle one timeout */
printf("\nMultiple Tasks waiting on Semaphore with middle one timeout:\n");
printf("Request Task 1 to wait on semaphore\n"); sendmsg(task1h,1);
printf("Request Task 2 to wait on semaphore after 1 second delay\n");sendmsg(task2h,3);
printf("main waiting on semaphore with timeout\n");
printf("\nReturn value to main was %d\n",swait(sema,40));
printf("\nSignalling semaphore twice\n");
      ssignal(sema);
      ssignal(sema);
pause();

/* Have two tasks wait on semaphore with first one timeout */
printf("\nTwo Tasks waiting on Semaphore with first one timeout:\n");
printf("Request Task 2 to wait on semaphore after 1 second delay\n");sendmsg(task2h,3);
printf("main waiting on semaphore with timeout\n");
printf("\nReturn value to main was %d\n",swait(sema,40));
printf("\nSignalling semaphore\n");
ssignal(sema);
pause();

/* Have two tasks wait on semaphore with second one timeout */
printf("\nTwo Tasks waiting on Semaphore with second one timeout:\n");
printf("Request Task 1 to wait on semaphore\n");sendmsg(task1h,1);
printf("main waiting on semaphore with timeout\n");
printf("\nReturn value to main was %d\n",swait(sema,40));
printf("\nSignalling semaphore\n");
ssignal(sema);
sclose(sema);
}

demoefwait()
{
/* Open locked event flag */
if ((eflag=efopen(0))==0) printf("No event flag available\n");
printf("\nDoing efwait() with timeout occurring\n"); 
printf("\nReturn value was %d\n",efwait(eflag,1,40));

putstr("\nDoing efwait() without timeout occurring\n"); efset(eflag, 1);
printf("\nReturn value was %d\n",efwait(eflag,1,40));

putstr("\nDoing efwait() with signal after one second\n");
sendmsg(task1h,5);/* Request efset after one second */
printf("\nReturn value was %d\n",efwait(eflag,1,40));

pause();
efclear(eflag,3);/* reset event flag*/
printf("\nMultiple Tasks waiting on eventflag with middle one timeout:\n");
printf("Request Task 1 to wait on eventflag\n");   sendmsg(task1h,6);
printf("Request Task 2 to wait on eventflag after 1 second delay\n");
                                                   sendmsg(task2h,6);
printf("main waiting on eventflag with timeout\n");
printf("\nReturn value to main was %d\n",efwait(eflag,1,40));
printf("\nSignalling eventflag with value 1\n"); efset(eflag,1);
printf("\nSignalling eventflag with value 2\n"); efset(eflag,2);
pause();
efclear(eflag,3);/* reset event flag*/

/* Have two tasks wait on event flag with first one timeout */
printf("\nTwo Tasks waiting on event flag with first one timeout:\n");
printf("Request Task 2 to wait on event flag after 1 second delay\n");
                                                   sendmsg(task2h,6);
printf("main waiting on event flag with timeout\n");
printf("\nReturn value to main was %d\n",efwait(eflag,1,40));
printf("\nSignalling event flag\n");
efset(eflag,3);
pause();

/* Have multiple tasks wait on event flag with last one timeout */
efclear(eflag,3);/* reset event flag*/
printf("\nMultiple Tasks waiting on event flag with last one timeout:\n");
printf("Request Task 1 to wait on event flag\n");sendmsg(task1h,6);
printf("Request Task 2 to wait on eventflag after 1 second delay\n");
                                                   sendmsg(task2h,7);
printf("main waiting on event flag with timeout\n");
printf("\nReturn value to main was %d\n",efwait(eflag,1,40));
printf("\nSignalling event flag\n"); efset(eflag,3);
efclose(eflag);
}

showmenu()
{
putstr("\n\n  MX-11 Example program 1\n\n");
putstr("  1            sleep()\n");
putstr("  2            gettime()\n");
putstr("  3            sendmsg()/getmsg()\n");
putstr("  4            sendmsg()/getmsg()\n");
putstr("  5            chkmsg()\n");
putstr("  6            opensem()\n");
putstr("  7            swait()\ssignal()\n");
putstr("  8            swait() timeout\n");
putstr("  9            efwait()\n");
putstr("  T            Show tasks\n\n");
putstr("Press selection,  press <ESC> to exit...");
}

char *decstatus(tstat)
int tstat; /*decode status field into pointer to string*/
{          /* Negative numbers correspond to timer functions */
if (tstat<0) return TStatus[tstat&15];
else         return Status [tstat];
}

showtasks()
{
/* Follow TCB's and display Tasks */
printf("MX Variables start at %4x\n", mxvar());
pTCB=mxvar();
printf(
"Task                TCB   Start Stack Status    Pri SPend SPmin Space\n");
printf(
"----                ---   ----- ----- ------    --- ----- ----- -----\n");

while (pTCB=pTCB->tlink)
   {
   printf("%-18s  %4x  %4x  %4x  %s  %d  %4x  %4x   %d\n",
   pTCB->tname, pTCB, pTCB->tstart, pTCB->tsp, decstatus(pTCB->tstat),
   pTCB->tpriority,pTCB->tspend,pTCB->tspmin,(pTCB->tspmin-pTCB->tspend));
   }
putch('\n');
}

main()
{
/* Initialize MX for 7 tasks, 32 MCB's and 256 byte stack space */
mainh=mxinit(256,7,32);
mxstkchk();

task1h=deftask( &task1,256,"Task 1"); /* define task1 */ 
task2h=deftask( &task2,256,"Task 2"); /* define task2 */ 
task3h=deftask( &task3,256,"Task 3"); /* define task3 */ 
/* Main now waits for Commands until escape pressed */

do {
   showmenu();
   c=getch(); putch('\n');
   switch (c) {
      case '1':demosleep(); break;
      case '2':demogettime(); break;
      case '3':demosendmsg(); break;
      case '4':demogetmsgt(); break;
      case '5':demochkmsg(); break;
      case '6':demoopensem(); break;
      case '7':demoswait(); break;
      case '8':demoswaitt(); break;
      case '9':demoefwait(); break;

      case 'T':
      case 't':showtasks(); break;
      }
if (c!=27) {putstr("\nPress any key to continue..."); getch();putch('\n');
   }
}
while (c !=27); /* Repeat until escape key pressed */
putstr("\nProgram finished\n");
sleep(20);
disable();
}
