/*
 * MX 68HC11 Multitasking Executive Example Program 4
 * Demonstrates interrupt routines written in C
 */
#include \mc\stdio.h
#include \mc\lib11\mx.h
#include ex.h

#define oc2i  0x40
#define oc2f  0x40
#define tcnt  0x100E
#define toc2  0x1018
#define tflg1 0x1023
#define tmsk1 0x1022

int error;
char *task1h, *task2h, *task3h, *task4h, *isrh, *mainh, *sema, c;
int  i,j,k, mode, mode1, latency, latmax, latmin;
int  allmax, allmin;

ISR()
{
latency=peekw(tcnt) - peekw(toc2);
poke(tflg1,oc2f);                          /* Reset interrupt flag   */
pokew(toc2,(peekw(toc2)+19999));           /* set for approx 100 Hz  */
enable();

latmax=max(latency, latmax);
latmin=min(latency, latmin);

switch(mode){
   case 1: ssignal(sema);     break;       /* Signal task 1          */
   case 2: sendmsg(task2h,1); break;       /* Send message to task 2 */
   }
}

task1()
{
for(;;){
   swait(sema,0);  /* Wait on semaphore */
   if (++i==100) { i=0; Cputc('1');}
   }
}

task2(message)
int message;
{
switch(message){
   case 1: if (++j == 100) { j=0; Cputc('2');} break;
   case 2: if (++k == 20)  { k=0; Cputc('3');} break;
   }
}

task3()
{
if (mode1) sendmsg(task2h,2);
}

outputlat()
{
Cprintf("\nAlltime maximum latency %5d\n",   allmax/2);
Cprintf("Alltime minimum latency %5d\n", allmin/2);
}

task4(message)
int message;
{
int i;
if ((mode==3) && (message==0)){
   i=0;
   if (allmax<latmax) { allmax=latmax; i++;}
   if (allmin>latmin) { allmin=latmin; i++;}
   if (i>0) outputlat();
   Cputc('.');
   }
if ((message==1) && (mode==3)) outputlat();
}

Startint()
{
Cputstr("\nEnabling Counter 2 interrupts \n");
disable();
poke(tmsk1,(peek(tmsk1)|oc2i)); /* Enable counter 2 interrupts */
enable();
}

Stopint()
{
Cputstr("\nDisabling Counter 2 interrupts \n");
disable();
poke(tmsk1,(peek(tmsk1)&~oc2i)); /* Disable counter 2 interrupts */
enable();
}

showmenu()
{
Cputstr("\n\n  MX-11 C Interrupt Example program\n\n");
Cputstr("  1            Start Interrupt routine using ssignal()\n");
Cputstr("  2            Start Interrupt routine using sendmsg()\n");
Cputstr("  3            Stop  Interrupt routine\n");
Cputstr("  4            Send messages to Task 2 at 1/20 second\n");
Cputstr("  5            Stop sending messages to Task 2\n");
Cputstr("  6            Measure Interrupt Latency\n");
Cputstr("  T            Show tasks\n\n");
Cputstr("Press selection,  press <ESC> to exit...");
}

showtasks()
{
/* Follow TCB's and display Tasks */
Cprintf("MX Variables start at %4x\n", mxvar());
pTCB=mxvar();
Cprintf(
"Task                TCB   Start Stack Status    Pri SPend SPmin Space\n");
Cprintf(
"----                ---   ----- ----- ------    --- ----- ----- -----\n");

while (pTCB=pTCB->tlink)
   {
   Cprintf("%-18s  %4x  %4x  %4x  %s  %d  %4x  %4x   %d\n",
   pTCB->tname, pTCB, pTCB->tstart, pTCB->tsp, Status[pTCB->tstat],
   pTCB->tpriority,pTCB->tspend,pTCB->tspmin,(pTCB->tspmin-pTCB->tspend));
   }
Cputc('\n');
}

main()
{
/* Initialize MX for 7 tasks, 32 MCB's and 256 byte stack space */
mainh=mxinit(256,7,32);
/*mxstkchk();*/
Copen(20,20);

sema=sopen(0);
mode1=0; mode=0; i=0; j=0; k=0;

latmax=0; latmin=2000;
allmax=0; allmin=2000;

task4h = deftask( &task4, 256, "Task 4"); /* define task4 */ 
task1h = deftask( &task1, 256, "Task 1"); /* define task1 */ 
task2h = deftask( &task2, 256, "Task 2"); /* define task2 */ 
task3h = deftask( &task3, 256, "Task 3"); /* define task3 */ 
isrh   = deftask( &ISR,   256, "ISR");    /* Define Interrupt routine   */

defintr         (  isrh, OC2);            /* Attach to Counter 2 vector */
defpoll(task3h,TICK,0);                   /* Attach to poll queue*/
defpoll(task4h,SECOND,0);                 /* Attach to poll queue*/
defpoll(task4h,MINUTE,1);                 /* Attach to poll queue*/

sendmsg(task1h,0);              /* Start up task1 to wait on semaphore */

/* Main now waits for Commands until escape pressed */
do {
   showmenu();
   c=Cgetc(); Cputc('\n');
   switch (c) {
      case '1':mode  = 1; Startint(); break;
      case '2':mode  = 2; Startint(); break;
      case '3':Stopint(); break;
      case '4':mode1 = 1; break; /* let task3 send messages to 2   */
      case '5':mode1 = 0; break; /* prevent task3 sending messages */
      case '6':Startint(); sleep(2);
               latmax=0; allmax=0; latmin=2000; allmin=2000;
               mode = 3;  sleep(20);
               outputlat();
               Cgetc(); Cputc('\n'); Stopint(); mode=0;
               break; /* Measure int Latency */
      case 'T':
      case 't':showtasks(); break;
      }
sleep(5);
if (c!=27) {Cputstr("\nmain(): Press any key to continue...");
            Cgetc();Cputc('\n');
   }
}
while (c !=27); /* Repeat until escape key pressed */
Cputstr("\nProgram finished\n");
sleep(20);
disable();
}
